@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -35,9 +35,20 @@ namespace block
 class BlockChart;
 class BlockInstruction;
 class BlockBlock;
+class MultiBlock;
 typedef UPtrList<BlockInstruction> BlockInstrList;
 typedef UPtrList<BlockBlock> BlockBlockList;
 
+/** Generates a full name for a block form the name given to it by the user
+ * and the full name of the parent. Rules are
+ * - Named blocks which are children of a named parent append their name with a dot.
+ * - Unnamed blocks' full name is the same as that of their parent plus a dot.
+ * - Unnamed blocks, whose parent is also unnamed (hence their name ends in a dot)
+ *   have the same name as their parent. A.k.a. we add no second dot.
+ * - Named blocks where parents are unnamed, just add the name (to avoid double dots).
+ *
+ * Note: 'name' may also contain dots, we apply the same rules here.*/
+std::string FormChildName(std::string_view parent, std::string_view name);
 
 /** A blockchart element: a block or an arrow.*/
 class BlockElement : public Element
@@ -49,7 +60,7 @@ public:
     using SpecBlockPtrs = std::array<const BlockBlock*, 5>;
     bool valid = true;              ///<Set to false, if constructor fails, the arrow endpoints dont exist or its path cannot be found. Drop this instruction in PostParseProcess()
     BlockChart    &chart;           ///<The block chart this element belongs to.
-    const std::string name_prefix;  ///<The prefix of our parent (without the dot). Empty only at top level.
+    const std::string name_prefix;  ///<Our prefix (without the dot): the full name of our parent (may be equal to the full name of its parent if our parent has no name). Empty only at top level.
     BlockStyle     style;           ///<The style of this element
     Label          parsed_label;    ///<The label processed and parsed into lines. Set in PostParseProcess()
     StringFormat   basic_format;    ///<The formatting at the start of the label. Set in PostParseProcess(), used in FinalizeLabels()
@@ -98,12 +109,17 @@ public:
     bool Resolve(str_pos_vector &blocks, SpecBlockPtrs spec_blocks,
                  std::string_view break_msg, std::string_view ambigous_msg,
                  std::string_view spec_block_msg, std::string *text, bool *justify);
+    struct HandleHidden {
+        enum HandleHiddenType { Keep, Remove, ToParent };
+        HandleHiddenType action;
+        std::string_view message = {}; //This message is displayed as an error if non-empty. Replace %B to the parent collapsed and %b to the block removed.
+    };
     template <typename str_pos_vector>
     bool CheckExists(str_pos_vector &blocks, std::string_view whattoignore,
-                     std::string_view self_msg = {}, bool keep_last = false);
+                     HandleHidden handle_hidden, std::string_view self_msg = {}, bool keep_last = false);
     virtual void MakeTemplate() { RemoveFromDrawOrder(); }
 
-    virtual bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number);
+    virtual bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number);
     virtual void FinalizeLabel(Canvas &canvas);
     void ShiftBy(const XY &xy) override { label_block.Shift(xy); Element::ShiftBy(xy); }
     void PostPosProcess(Canvas &canvas, Chart *ch) override;
@@ -111,7 +127,7 @@ public:
     void RegisterLabels() override;
     Contour LabelCover() const;
     void DrawLabel(Canvas &canvas) const;
-    virtual void Draw(Canvas &canvas) const {}
+    virtual void Draw(Canvas&) const {}
     ~BlockElement() override { RemoveFromDrawOrder(); }
 };
 
@@ -350,7 +366,7 @@ struct CloneAction
         action(ECloneContentAdjustment::REPLACE), action_pos(l), instr(inst), blocks({{blk, blk_pos}}) {}
     explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs) :
         action(ECloneContentAdjustment::DROP), action_pos(l) { if (instrs) blocks.swap(*instrs); delete instrs; }
-    explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs, const FileLineColRange &inst_pos,
+    explicit CloneAction(const FileLineColRange &l, gsl::owner<StringWithPosList*> instrs, const FileLineColRange &/*inst_pos*/,
                          gsl::owner<AttributeList*> a, gsl::owner<CloneActionList *>mod) :
         action(ECloneContentAdjustment::UPDATE), action_pos(l), attr(a), modifiers(mod) {if (instrs) blocks.swap(*instrs); delete instrs;}
 
@@ -389,7 +405,8 @@ public:
     BlockInstruction(const BlockInstruction &) = default;
     BlockInstruction(BlockInstruction &&) = default;
     virtual BlockInstruction *Clone(const FileLineColRange &l, std::string_view prefix,
-                                    gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const = 0;
+                                    gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                                    const MultiBlock *multi_parent) const = 0;
     /** Call this to apply the running style (appropriate parts) to this instruction.
      * @param [in] cloning True if this is applied during cloning.
      * @param [in] recursive If true, blocks will call this for all their content.*/
@@ -400,25 +417,25 @@ public:
     virtual void IncLevel() { }
     /** Do we act as content for a block, requiring our parent to accomodate us in it size?*/
     virtual bool AmIContent() const { return false; }
-    virtual Contour GetContourForArrows(const std::vector<XY> &except, bool skip_allow_arrows,
-                                        const StringWithPosList &except_blocks,
-                                        const StringWithPosList &except_parents,
-                                        double distance,
-                                        const StringWithPosNumberList &distance_per_block) const {return {};}
-    virtual bool IsMyChild(const std::unordered_set<const BlockBlock*> &except) const { return false; }
+    virtual Contour GetContourForArrows(const std::vector<XY> &/*except*/, bool /*skip_allow_arrows*/,
+                                        const StringWithPosList &/*except_blocks*/,
+                                        const StringWithPosList &/*except_parents*/,
+                                        double /*distance*/,
+                                        const StringWithPosNumberList &/*distance_per_block*/) const { return {}; }
+    virtual bool IsMyChild(const std::unordered_set<const BlockBlock*> &/*except*/) const { return false; }
     /** When we or a parent of us is made a template, here can we unregister any drawing
     * or spacing.*/
-    virtual void ResolveAlignmentAttributes(const BlockBlock *first, const BlockBlock *prev, const BlockBlock *next, const BlockBlock *last) { }
-    virtual void AddVariableIndices(ConstraintSet &constraints, unsigned from, unsigned to) const {}
+    virtual void ResolveAlignmentAttributes(const BlockBlock */*first*/, const BlockBlock */*prev*/, const BlockBlock */*next*/, const BlockBlock */*last*/) { }
+    virtual void AddVariableIndices(ConstraintSet&, unsigned /*from*/, unsigned /*to*/) const {}
     virtual AddConstraintResult
-        AddConstraints(Canvas & canvas, ConstraintSet &constraints, unsigned from, unsigned to,
-                       unsigned parent_index_justify[2], const InternalAlignment *parent_attr_justify[2]) const
-        {return {};}
-    virtual bool CompareLayouts(ConstraintSet &constraints, unsigned from, unsigned to) { return false; }
-    virtual void LayoutFromVariables(GLPKSolver &constraints, unsigned from, unsigned to) { }
-    virtual bool LayoutWithGLPK(Canvas &canvas, unsigned from, unsigned to, size_t progress_units) { return false; }
-    virtual void ShiftToZeroIfLaidOut(unsigned from, unsigned to) noexcept { }
-    virtual void LayoutFinalizeBlocks(Canvas & canvas, ELayoutContentFinalizePass p) {}
+        AddConstraints(Canvas&, ConstraintSet&, unsigned /*from*/, unsigned /*to*/,
+                       unsigned /*parent_index_justify*/[2], const InternalAlignment */*parent_attr_justify*/[2]) const
+        { return {}; }
+    virtual bool CompareLayouts(ConstraintSet&, unsigned /*from*/, unsigned /*to*/) { return false; }
+    virtual void LayoutFromVariables(GLPKSolver &/*constraints*/, unsigned /*from*/, unsigned /*to*/) { }
+    virtual bool LayoutWithGLPK(Canvas&, unsigned /*from*/, unsigned /*to*/, size_t /*progress_units*/) { return false; }
+    virtual void ShiftToZeroIfLaidOut(unsigned /*from*/, unsigned /*to*/) noexcept { }
+    virtual void LayoutFinalizeBlocks(Canvas&, ELayoutContentFinalizePass) {}
 };
 
 /** A class to store join commands*/
@@ -433,9 +450,10 @@ public:
     static bool AttributeValues(std::string_view attr, Csh &csh);
     void AddAttributeList(gsl::owner<AttributeList*>a) override;
     BlockInstruction *Clone(const FileLineColRange &l, std::string_view prefix,
-                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
-    void ApplyRunningStyle(bool cloning, bool recursive) override {}
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                            const MultiBlock* multi_parent) const override;
+    void ApplyRunningStyle(bool /*cloning*/, bool /*recursive*/) override {}
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
     void LayoutFinalizeBlocks(Canvas & canvas, ELayoutContentFinalizePass p) override;
     void Draw(Canvas &c) const override;
 };
@@ -457,9 +475,10 @@ public:
         BlockInstruction(chart, FileLineColRange()),
         set_length(len), inc_by(inc) {}
     BlockInstruction *Clone(const FileLineColRange &l, std::string_view prefix,
-                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
-    void ApplyRunningStyle(bool cloning, bool recursive) override {}
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+                            gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                            const MultiBlock* multi_parent) const override;
+    void ApplyRunningStyle(bool /*cloning*/, bool /*recursive*/) override {}
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
 };
 
 /** A block of a blockchart. May contain other blocks.
@@ -482,13 +501,13 @@ public:
  * - In LayoutFromVariables() we assume the layout has been solved and all variables have good values.
  *     Here we take these values and fill into 'outer_line' and 'label_block'.
  * - In PostPosProcess() we create our coverage area and register us for tracking.
- * - In Draw() we draw ourself.*/
+ * - In Draw() we draw ourselves.*/
 class BlockBlock : public BlockInstruction
 {
 protected:
     bool names_registered = false;      ///<Flag if we haves our names registered, so we when we unregister templates, they are not attempted to be registered
     BlockBlock(BlockChart &chart, EBlockType t, EBlockType c_t, int sh, const FileLineColRange &l, std::string_view name);
-    BlockBlock(const BlockBlock &o, const FileLineColRange &l, std::string_view name, std::string_view prefix);
+    BlockBlock(const BlockBlock &o, const FileLineColRange &l, std::string_view name, std::string_view prefix, bool copy_content=true);
     static AlignmentAttr *
         TranslatePre(BlockChart &chart, AlignModifier c1, AlignModifier c2,
                      const FileLineCol &l1, const FileLineCol &l2,
@@ -516,13 +535,22 @@ public:
     const std::string  name_full;       ///<The name of the block with its prefixes added. Empty if the user specified no name.
     const std::string  name_unique;     ///<The name of the block. We append the line number to make it unique.
           std::string  name_display;    ///<The human-readable name of the block. Only has line numbers for disambiguation, never empty. We disambiguate in PostParseProcess(), when we have all blocks added.
+          std::string  name_gui_collapse;///<Use this name when identifying us towards the GUI for the purpose of collapse/expand. This can be the name of another entity (if we are the front element of a multi).
 
     bool               track = true;    ///<If set to false, we do not add ourselves to the tracking rectangles (a child of multi)
     unsigned           level = 0;       ///<Our depth in the block hierarchy. Top level main node is 0.
     unsigned           rank = 0;        ///<The position among our parent's children. Used to break alignment attr ties. Set in SetContent() or ApplyCloneModifiers() of the parent
-    BlockInstrList     content;         ///<The content of the block, if any
+    BlockInstrList     content;         ///<The content of the block, as it shows (if any). For collapsed blocks this is either empty or holds an indicator.
+    BlockInstrList     collapsed_content;///<The content of the block, if collapsed.
     StringWithPosList  around;          ///<We should be around these blocks
-    bool    has_visible_content = false;///<True if we have any blocks within 'content' or 'around'. False if only other instructions (arrows, joins, etc.) are there. Set in SetContent().
+    bool orig_has_visible_content = false;///<True if we have any blocks within 'content' or 'around'. False if only other instructions (arrows, joins, etc.) are there. Set in SetContent().
+    bool      has_visible_content = false;///<True if we have any blocks within 'content' or 'around' *and* we are not collapsed. It can also be true if we are collapsed but show an indicator.
+    OptAttr<bool>      collapsed;       ///<The `collapsed=` attribute if specified either in source or via the GUI. Reset from 'orig_collapsed' on a clone (GUI commands not cloned.)
+    OptAttrLine<bool>  orig_collapsed;  ///<The `collapsed=` attribute if specified in source. `file_pos` contains the location of the attribute (name) in the input file.
+    const BlockBlock*  hidden_by = nullptr; ///<The topmost ancestor of us that is collapsed. If set, we are not shown.
+    std::optional<BlockStyle> alternate_style; ///<Set only if we have visible content and the indicator is false. If we are not collapsed, this contains the style for the collapsed us (and vice versa). Used when we are cloned with a different collapse status.
+    const StyleCoW<BlockStyle> indicator_style;///<The "indicator" style captured at the point of creation
+    const StyleCoW<BlockStyle> running_style;  ///<The block running style captured at the point of creation
 
     mutable unsigned index_out_min[2] = {};     ///<The GLPK variable index for left and top outer_line ignoring margin
     mutable unsigned index_out_max[2] = {};     ///<The GLPK variable index for right and bottom outer_line ignoring margin
@@ -572,6 +600,8 @@ public:
     bool IsApplicableAsPrevNext() const override { return around.size()==0; }
     /** Can we be multi'ed?*/
     virtual bool CanBeMulti() const { return true; }
+    /** Returns empty if can be collapsed, else an error to display for the attr. */
+    virtual std::string_view CanBeCollapsed() const noexcept { return is_any_of(content_type, EBlockType::Box, EBlockType::Boxcol, EBlockType::Shape) ? "" : "Only visible blocks can be collapsed."; }
     /** Get the location of a port.*/
     std::optional<Port> GetPort(std::string_view name) const;
     BlockBlock *FindApplicableNextAtOrAfter(BlockInstrList::const_iterator i) const;
@@ -610,13 +640,16 @@ public:
     void ApplyPre(gsl::owner<AlignmentAttr *>local_alignment);
 	virtual void ApplyFurtherAttributes(const AttributeList *al);
     void AddAround(gsl::owner<StringWithPosList*> blocks, const FileLineCol &l);
+    virtual void UpdateCollapsedFromGUI();
     void ApplyCloneModifiers(gsl::owner<CloneActionList *> mod);
     BlockBlock *Clone(const FileLineColRange &l, std::string_view prefix,
-					  gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override
-        { return CloneAs(l, name_original, prefix, nullptr, attr, mod); }
+					  gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                      const MultiBlock* multi_parent) const override
+        { return CloneAs(l, name_original, prefix, nullptr, attr, mod, multi_parent); }
     virtual BlockBlock *CloneAs(const FileLineColRange &l, std::string_view name, std::string_view prefix,
 								gsl::owner<AlignmentAttr*> align,
-								gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const;
+								gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                                const MultiBlock* multi_parent) const;
     void ApplyRunningStyle(bool cloning, bool recursive) override;
     BlockBlock *GetCommonParent(const BlockBlock *other);
     const BlockBlock *GetCommonParent(const BlockBlock *other) const;
@@ -631,7 +664,7 @@ public:
     bool IsMyParent(const BlockBlock *p) const { return p && (p==parent || (parent && parent->IsMyParent(p))); }
     bool IsMyChild(const std::unordered_set<const BlockBlock*> &except) const override;
     void KillAlignmentBasedOnChildAttr(bool y, EAlignPrio p);
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
     void FinalizeLabel(Canvas &canvas) override;
     void ResolveAlignmentAttributes(const BlockBlock *first, const BlockBlock *prev, const BlockBlock *next, const BlockBlock *last) override;
     void AddVariableIndices(ConstraintSet &constraints, unsigned from, unsigned to) const override;
@@ -673,10 +706,12 @@ public:
         BlockBlock(ch, EBlockType::Space, l), size(s) {}
     BlockBlock *CloneAs(const FileLineColRange &l, std::string_view name, std::string_view prefix,
 						gsl::owner<AlignmentAttr*> align,
-						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
+						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                        const MultiBlock* multi_parent) const override;
 
     bool CanBeMulti() const override { return false; }
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+    std::string_view CanBeCollapsed() const noexcept override { return "Spaces cannot be collapsed."; }
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
 };
 
 /** A block resulting from a "break" command.*/
@@ -690,9 +725,27 @@ public:
         BlockBlock(ch, EBlockType::Space, l) {}
     BlockBlock *CloneAs(const FileLineColRange &l, std::string_view name, std::string_view prefix,
 						gsl::owner<AlignmentAttr*> align,
-						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
+						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                        const MultiBlock* multi_parent) const override;
     bool CanBeMulti() const override { _ASSERT(0); /*parser should not get here*/ return false; }
-    bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+    std::string_view CanBeCollapsed() const noexcept override { return "Breaks cannot be collapsed."; }
+    bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
+};
+
+/** A block showing as an indicator. Used only internally.*/
+class BlockIndicator : public BlockBlock {
+public:
+    BlockIndicator(BlockChart& ch, const FileLineColRange& l) :
+        BlockBlock(ch, EBlockType::Indicator, l) {}
+    BlockBlock* CloneAs(const FileLineColRange&, std::string_view /*name*/, std::string_view /*prefix*/,
+                        gsl::owner<AlignmentAttr*>,
+                        gsl::owner<AttributeList*>, gsl::owner<CloneActionList*>,
+                        const MultiBlock* /*multi_parent*/) const override { _ASSERT(0); return nullptr; }
+    bool CanBeMulti() const override { _ASSERT(0); /*parser should not get here*/ return false; }
+    std::string_view CanBeCollapsed() const noexcept override { return "Indicators cannot be collapsed."; }
+    void SetContent(BlockInstrList* c) override;
+    bool PostParseProcess(Canvas& canvas, const BlockBlock* hide_by, Numbering& number) override;
+    void Draw(Canvas& canvas) const override;
 };
 
 class MultiBlock : public BlockBlock
@@ -701,20 +754,24 @@ class MultiBlock : public BlockBlock
 protected:
     MultiBlock(const MultiBlock &o, const FileLineColRange &l,
                std::string_view name, std::string_view prefix) :
-        BlockBlock(o, l, name, prefix), count(o.count) {}
+        BlockBlock(o, l, name, prefix, /*copy_content=*/false), count(o.count) {}
 
 public:
     const unsigned count; ///<How many copies to show
     /** Constructor to initialize a row, column, box or space - any block other than a shape.*/
     MultiBlock(int num, gsl::owner<BlockBlock *> b, const FileLineColRange &l);
+    static int CategorizeAttribute(const Attribute& a);
+    static AttributeList* CopyAttributes4Children(const AttributeList* al, bool front);
 	void ApplyFurtherAttributes(const AttributeList *al) override;
 	BlockBlock *CloneAs(const FileLineColRange &l, std::string_view name, std::string_view prefix,
 						gsl::owner<AlignmentAttr*> align,
-						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod) const override;
+						gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList *> mod,
+                        const MultiBlock* multi_parent) const override;
     void ApplyRunningStyle(bool cloning, bool recursive) override;
     bool CanBeMulti() const override { _ASSERT(0); /*parser should not get here*/ return false; }
+    void UpdateCollapsedFromGUI() override {} //Special handling of this in MultiBlock::SetContent()
     void SetContent(BlockInstrList *c) override;
-	bool PostParseProcess(Canvas &canvas, bool hide, Numbering &number) override;
+	bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
 	void LayoutFinalizeBlocks(Canvas & canvas, ELayoutContentFinalizePass p) override;
 };
 
@@ -724,17 +781,17 @@ public:
 struct CopyParseHelper
 {
     FileLineColRange file_pos; ///<The position of the header
-    char *block_name;          ///<The block we copy (may be null if it is omitted - ignore command then)
+    multi_segment_string block_name; ///<The block we copy (may be null if it is omitted - ignore command then)
     FileLineCol block_pos;     ///<The location of the name of the to-be copied block.
     const BlockBlock *block;   ///<The block we copy. We do not own this pointer.
-    char *copy_name;           ///<The name of the copy (if any)
+    multi_segment_string copy_name; ///<The name of the copy (if any)
     AlignmentAttr *align;      ///<Any alignment modifiers
     AttributeList *attributes; ///<Updated/added attributes of the copy (if any)
     CloneActionList *modifiers;///<Updated/added content of the copy (if any)
     /** Initialize to all nulls.*/
-    void Init() { file_pos.MakeInvalid(); block_name = nullptr; block_pos.MakeInvalid(); block = nullptr; copy_name = nullptr; align = nullptr;  attributes = nullptr; modifiers = nullptr; }
+    void init() { file_pos.MakeInvalid(); block_name.init(); block_pos.MakeInvalid(); block = nullptr; copy_name.init(); align = nullptr;  attributes = nullptr; modifiers = nullptr; }
     /** Free non-null members */
-    void Free() { free(block_name); free(copy_name); delete align;  delete attributes; delete modifiers; Init(); }
+    void destroy() { block_name.destroy(); copy_name.destroy(); delete align;  delete attributes; delete modifiers; init(); }
 };
 
 inline unsigned ConstraintSet::AddVariable(const BlockBlock *b, std::string_view type) {
