@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -101,9 +101,9 @@ public:
     /** Call this when parsing has started .*/
     virtual void StartParse() {}
     /** Call this to register a file to parse. */
-    virtual void RegisterParse(size_t len) {}
+    virtual void RegisterParse(size_t) {}
     /** Call this when you have parsed this much of the input. */
-    virtual void DoneParse(size_t len) {}
+    virtual void DoneParse(size_t) {}
     /** Call this when done.*/
     virtual void Done() { active = false; } //No more calls to Progress methods will have any effect (possible to re-draw)
     /** Call this when reset. Keep load data, but delete all collected data*/
@@ -115,7 +115,7 @@ public:
      * This function must tolerate empty or null input.
      * @returns True if OK, false if not. In the latter case load data
      *          is substituted with the defaults.*/
-    virtual bool ReadLoadData(std::string_view in) { return false; }
+    virtual bool ReadLoadData(std::string_view) { return false; }
 
     /** Return the number of unified sections we have (bulk and arc based in Progress).
      * The term unified section is used to hide differences between any kind of
@@ -125,7 +125,7 @@ public:
     /** Return the number of the current unified section we are in.*/
     virtual std::optional<unsigned> GetCurrentUnifiedSection() const { return {}; }
     /** return the user friendly name of a given section (if any).*/
-    virtual std::string_view GetUnifiedSectionName(unsigned section) const { return {}; }
+    virtual std::string_view GetUnifiedSectionName(unsigned) const { return {}; }
     /** Returns the percentage spent in each unified section so far. Should contain at most as many
      * values as GetSectionNum() - missing values assumed to be zero.*/
     virtual std::vector<double> GetUnifiedSectionPercentage() const { return {}; }
