@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -47,7 +47,7 @@ void DeserializeWhiteSpace(std::string_view &s)
 bool DeserializeToken(std::string_view &s, std::string_view token)
 {
     DeserializeWhiteSpace(s);
-    if (s.substr(0, token.length()) != token) 
+    if (s.substr(0, token.length()) != token)
         return false;
     s.remove_prefix(token.length());
     return true;
@@ -103,7 +103,7 @@ std::pair<bool, double> DeserializeDouble(std::string_view &s)
     DeserializeWhiteSpace(s);
     char *end;
     double d = strtod(s.data(), &end);
-    if (end==s.data()) 
+    if (end==s.data())
         return {false, {}};
     s.remove_prefix(end-s.data());
     return {true, d};
@@ -121,9 +121,9 @@ std::vector<int64_t> DeserializeInts(std::string_view &s, size_t maxnum)
     while(ret.size()<maxnum) {
         DeserializeWhiteSpace(s);
         auto r = DeserializeInt(s);
-        if (r.first) 
+        if (r.first)
             ret.push_back(r.second);
-         else 
+         else
             break;
     }
     return ret;
@@ -141,9 +141,9 @@ std::vector<double> DeserializeDoubles(std::string_view &s, size_t maxnum)
     while(ret.size()<maxnum) {
         DeserializeWhiteSpace(s);
         auto r = DeserializeDouble(s);
-        if (r.first) 
+        if (r.first)
             ret.push_back(r.second);
-         else 
+         else
             break;
     }
     return ret;
@@ -190,19 +190,19 @@ bool ProgressBase::DoCallback(double p) const
 
 const char LabelInfo::labelTypeChar[] = "-EADBbPVCSN";
 
-std::unique_ptr<Csh> Chart::CshFactory(Csh::FileListProc proc, void *param) const
+std::unique_ptr<Csh> Chart::CshFactory(Csh::FileListProc) const
 {
     return std::make_unique<DummyCsh>();
 }
 
 /** Find what fill to apply in the range Y.We assume all fill blocks span the
- * entire chart width and only differ in their Y coordinate. We search for the fill block 
+ * entire chart width and only differ in their Y coordinate. We search for the fill block
  * covering 'y' (or is at least adjacent to it) and we synthetize something close.
- * This is needed because the footer (copyright text) is drawn genericly and may be 
- * at the middle of the chart (if at page boundary). Each individual language must 
+ * This is needed because the footer (copyright text) is drawn generically and may be
+ * at the middle of the chart (if at page boundary). Each individual language must
  * expose its background so that it can be drawn with the correct background.
  * @param [in] Y The y range we generate the fill for in chart coordinates
- * @param [in] top If true, then Y shall be synthetized from the fill at Y.till 
+ * @param [in] top If true, then Y shall be synthetized from the fill at Y.till
  *             meaning the filled region is a header. If false it is a footer and the
  *             fill at Y.from shall be used to synthetize the result.
  *
@@ -217,7 +217,7 @@ FillAttr Chart::GetBkFill(Range Y, bool top) const noexcept {
     for (const BackgroundFill &f : bkFill)
         switch (f.area.y.IsWithin(y)) {
         default:
-        case contour::WI_IN_HOLE: 
+        case contour::WI_IN_HOLE:
         case contour::WI_OUTSIDE: continue;
         case contour::WI_INSIDE: best = good = &f; break;
         case contour::WI_ON_EDGE:
@@ -236,7 +236,7 @@ FillAttr Chart::GetBkFill(Range Y, bool top) const noexcept {
 Chart::Chart(FileReadProcedure *p, void *param) :
     first_include(-1),
     total(0, 0, 0, 0), copyrightTextHeight(0),
-    noLabels(0), noOverflownLabels(0), 
+    noLabels(0), noOverflownLabels(0),
     file_read_proc(p), file_read_proc_param(param)
 {
     ignore_designs = false;
@@ -259,7 +259,7 @@ const double Chart::compass_point_degree[] =
  * We return -2 on perpendicular to contour.*/
 std::optional<double>
 Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
-                         bool report_error) 
+                         bool report_error)
 {
     double ret;
     if (!from_chars(n, ret)) {
@@ -305,7 +305,7 @@ Chart::ParseCompassPoint(std::string_view n, const FileLineCol &l,
 * @param [in] margins For fixed size pages, these are the margins
 * @param [in] ha The horizontal alignment for fix size pages (-1: left, 0: center, +1:right)
 * @param [in] va The vertical alignment for fix size pages (-1:up, 0: center, +1:bottom)
-* @param [in] generateErrors If true and we cannot avoid overfill, we geenrate a warining.
+* @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
 *                            Errors are also generated on file creation and similar hard errors.
 * @returns False on error (but not on warning) irrespective of `generateErrors`. */
 bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
@@ -345,12 +345,12 @@ bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
             } else {
                 Canvas canvas(ot, total, copyrightTextHeight, fn, scale, &pageBreakData, page);
                 if (canvas.ErrorAfterCreation(generateErrors ? &Error : nullptr, &pageBreakData)) return false;
-                if (include_chart_text) 
+                if (include_chart_text)
                     canvas.AddiTXtChunk(EmbedChartData{ .chart_type = GetLanguageExtensions().front(), .chart_text = include_chart_text,
                                                         .chart_size = GetCanvasSize(), .gui_state=SerializeGUIState(), .page=page});
                 DrawComplete(canvas, bPageBreak, page);
             }
-        } 
+        }
     else {
         _ASSERT(ot!=Canvas::ISMAP);
         _ASSERT(ot==Canvas::PDF || ot==Canvas::PPT); //only PDF/PPT supports multiple pages.
@@ -376,7 +376,7 @@ bool Chart::DrawToFile(Canvas::EOutputType ot, const XY &scale,
 * @param [in] fallback_image_resolution Draw fallback images at this resolution.
 * @param [out] metafile_size Return the size of the resultant metafile in bytes
 * @param [out] fallback_images Return the area covered by fallback images (in chart space)
-* @param [in] generateErrors If true and we cannot avoid overfill, we geenrate a warining.
+* @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
 *                            Errors are also generated on file creation and similar hard errors.
 * @returns The EMF handle of the metafile created. */
 HENHMETAFILE Chart::DrawToMetaFile(Canvas::EOutputType ot,
@@ -410,7 +410,7 @@ HENHMETAFILE Chart::DrawToMetaFile(Canvas::EOutputType ot,
 * @param [in] bPageBreaks If true and we draw the whole chart and the chart has multiple
 *                         pages, draw dashed lines for page breaks.
 * @param [in] fallback_image_resolution Draw fallback images at this resolution.
-* @param [in] generateErrors If true and we cannot avoid overfill, we geenrate a warining.
+* @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
 *                            Errors are also generated on file creation and similar hard errors.
 * @returns The size of the metafile or zero at error.*/
 size_t Chart::DrawToDC(Canvas::EOutputType ot, HDC hdc, const XY &scale,
@@ -437,7 +437,7 @@ size_t Chart::DrawToDC(Canvas::EOutputType ot, HDC hdc, const XY &scale,
  * @param [in] ot The format of output. Determines what type of approximations we use.
  * @param [in] bPageBreaks If true and the chart has multiple
  *                         pages, draw dashed lines for page breaks.
- * @param [in] generateErrors If true and we cannot avoid overfill, we geenrate a warining.
+ * @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
  *                            Errors are also generated on file creation and similar hard errors.
  * @returns The resultant cairo recording surface.
  */
@@ -463,10 +463,10 @@ cairo_surface_t *Chart::DrawToRecordingSurface(Canvas::EOutputType ot, bool bPag
 /** Replay the chart or one page of it onto a cairo recording surface using a version previously
 * recorded via DrawToRecordingSurface().
 * This function is used when the user views just one page.
-* @param [in] full A recording suface onto which we previously recorded the drawing
+* @param [in] full A recording surface onto which we previously recorded the drawing
 *                  of the chart using DrawToRecordingSurface().
 * @param [in] page The page to draw, zero for the whole chart.
-* @param [in] generateErrors If true and we cannot avoid overfill, we geenrate a warining.
+* @param [in] generateErrors If true and we cannot avoid overfill, we generate a warning.
 *                            Errors are also generated on file creation and similar hard errors.
 * @returns The resultant cairo recording surface.
 */
@@ -493,10 +493,10 @@ cairo_surface_t *Chart::ReDrawOnePage(cairo_surface_t *full, unsigned page,
     return ret;
 }
 
-// Leaves ignore_designs and prepare_for_tracking intact.  
+// Leaves ignore_designs and prepare_for_tracking intact.
 // Leaves copyright text and its height intact.
 // We also keep errors.
-// but deletes shapes 
+// but deletes shapes
 void Chart::SetToEmpty()
 {
     total = Block(0, 0, 0, 0);
@@ -512,7 +512,7 @@ void Chart::SetToEmpty()
     used_shapes.clear();
 }
 
-std::pair<const std::string*, FileLineCol> 
+std::pair<const std::string*, FileLineCol>
 Chart::Include(std::string_view filename, const FileLineCol &linenum_command)
 {
     std::pair<const std::string*, FileLineCol> ret; //empty string, invalid file pos
@@ -547,20 +547,20 @@ XY Chart::GetCanvasSize(unsigned page) const noexcept {
 /** Ensure that the attribute value specifies a color.
  *
  * If the attribute value is not a string that conforms to color specifications
- * (that is, not a name of a color and not an rgb value 
+ * (that is, not a name of a color and not an rgb value
  * triplet, etc., see ColorSet::GetColor()) we generate an error into `error` and
- * return false. 
+ * return false.
  * If the current context ignores colors/styles, we omit generating the error.
  * @param [in] a The attribute we test.
  * @returns True if the value specifies a color.*/
-bool Chart::CheckColor(const Attribute & a) 
+bool Chart::CheckColor(const Attribute & a)
 {
     if (a.type==EAttrType::STRING) {
         ColorType c = GetCurrentContext()->colors.GetColor(a.value);
         if (c.type!=ColorType::INVALID) return true;
     }
     if (!GetCurrentContext()->SkipContent())
-        Error.Error(a, true, "Unrecognized color name or definition: '" + 
+        Error.Error(a, true, "Unrecognized color name or definition: '" +
                     StringFormat::RemovePosEscapesCopy(a.value.c_str()) + "'. Ignoring attribute.");
     return false;
 }
