@@ -1,6 +1,6 @@
 /*
   This file is part of Msc-generator.
-Copyright (C) 2008-2022 Zoltan Turanyi
+Copyright (C) 2008-2023 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -46,24 +46,10 @@ struct ProcDefParseHelper
     bool                          had_error;
     FileLineCol                   linenum_name;
     FileLineCol                   linenum_body;
-    ProcDefParseHelper() : name(nullptr), parameters(nullptr), attrs(nullptr), body(nullptr), had_error(false) {}
-    ~ProcDefParseHelper() { free(name.str);   delete parameters; delete attrs; delete body; }
+    ProcDefParseHelper() : parameters(nullptr), attrs(nullptr), body(nullptr), had_error(false) { name.init(); }
+    ~ProcDefParseHelper() { name.destroy(); delete parameters; delete attrs; delete body; }
 };
 
-/** Structure making parsing procedure definitions simpler.*/
-template<class AttrList>
-struct CppProcDefParseHelper
-{
-    MultiString                     name;       ///<The name of the procedure. Must be there and non-empty for a valid definition.
-    std::optional<ProcParamDefList> parameters; ///<The parameter list of the procedure. If not there, we had a problem with the parameter list.
-    AttrList                        attrs;      ///<The attribute list of the definition (e.g., [export=yes]). May be empty.
-    std::unique_ptr<Procedure>      body;       ///<The body. For a valid procedure we must have one status=OK. If null it signals an error (missing body).
-    bool                            had_error = false;
-    FileLineCol                     linenum_name;
-    FileLineCol                     linenum_body;
-};
-
-
 
 /** Holds coordinate info about one element or entity label.
  * Used to generate 'lmap' output.
@@ -143,7 +129,7 @@ public:
     virtual std::string GetLanguageDefaultText() const = 0; ///<Returns the default text of the chart in UTF-8
     virtual bool GetLanguageHasAutoheading() const = 0; ///<True if the chart has automatic headings
     virtual bool GetLanguageHasElementControls() const = 0; ///<True if the elements of the chart support GUI controls
-    virtual std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, void *param) const; ///<Return the corresponding color syntaxer
+    virtual std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const; ///<Return the corresponding color syntax highlighter
     virtual std::map<std::string, std::string> RegisterLibraries() const { return {}; } ///<Return the libraries used (for the purpose of the GUI about box and commandline --version flag. In UTF-8, like "cairo v1.2.3". No need to register cairo, Msc will do that.
 
     /** @} */
@@ -208,14 +194,14 @@ public:
     /** Sets the default font and font language. Leave empty if none.
      * You does not have to call it, but if you do, do it right after the
      * creation of the chart object, before parsing.
-     * For signalling charts and block diagrams, this is synonimous with
+     * For signalling charts and block diagrams, this is synonymous with
      * setting the text.font.{face,lang} chart options, for graphs it is
-     * synonimous with issuing the {graph,node,edge} [ fontname = <face> ];
+     * synonymous with issuing the {graph,node,edge} [ fontname = <face> ];
      * commands (and lang is ignored).*/
     virtual void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange &l) = 0;
 
     /** Opens and loads a file for inclusion. Filename is UTF-8.
-     * Returns the text to include and the poistion of its first character.*/
+     * Returns the text to include and the position of its first character.*/
     virtual std::pair<const std::string*, FileLineCol> Include(std::string_view filename, const FileLineCol &linenum_command);
 
     /** Just before compiling a chart (after filling in the designs from the designlib) apply the forced design.
@@ -258,14 +244,14 @@ public:
     bool Reparsing() const { return GetCurrentContext() ? GetCurrentContext()->Reparsing() : false;  }
     /** True if we (based on the current context) are parsing a proc definition do not check names.*/
     bool SkipContent() const { return GetCurrentContext() ? GetCurrentContext()->SkipContent() : false; }
-    /** Look up a procedure by name. Used during parsing and we seach the whole context stack.*/
-    virtual const Procedure *GetProcedure(const std::string &name) const = 0;
-    /** Look up a parameter or variable by name. Used during parsing and we seach the context stack:
+    /** Look up a procedure by name. Used during parsing and we search the whole context stack.*/
+    virtual const Procedure *GetProcedure(std::string_view name) const = 0;
+    /** Look up a parameter or variable by name. Used during parsing and we search the context stack:
      * - for parameters down to and including the context defining or replaying a procedure;
      * - for variables all the way.*/
-    virtual ProcParamResolved *GetParameter(const std::string &name) = 0;
+    virtual ProcParamResolved *GetParameter(std::string_view name) = 0;
     /** Get a particular reference. Valid after ParseText().*/
-    virtual const RefNameData *GetReference(const std::string &refname) const = 0;
+    virtual const RefNameData *GetReference(std::string_view refname) const = 0;
     /** Get value of the "width" chart option - this is msc specific, -1 if none. Valid after ParseText().*/
     virtual double GetWidthAttr() const { return -1; } //msc specific
     /** Deserialize the GUI state (expand/collapse attributes) from a string. Empty string always ok and means no GUI exceptions.*/
@@ -289,7 +275,7 @@ public:
     /** Adds the list of chart options to a Csh object. */
     static void AttributeNames(Csh &, bool /*designOnly*/) {}
     /** Adds the list valid values for a chart options to a Csh object. */
-    static bool AttributeValues(const std::string /*attr*/, Csh &/*csh*/) { return true; }
+    static bool AttributeValues(std::string_view /*attr*/, Csh &/*csh*/) { return true; }
     virtual ProgressBase *GetProgress()=0;   ///<Get the element tracking our progress
 
     /** Parses a compass point to a degree.*/
@@ -301,7 +287,7 @@ public:
      * ready to accept new blocks of top-level syntax. (We will not break
      * top level blocks or instructions into two files, but may parse
      * several design libs in one go, and then the user input after; or this may be
-     * used later to implement \#inlcude-like behaviour.
+     * used later to implement \#include-like behavior.
      * It must maintain a ready copy of 'Shapes' and 'ChartBase::Designs'.
      * @param [in] input string of UTF8 encoded chart text
      * @param [in] filename The UTF8 name of the file to use in error messages.
@@ -315,7 +301,7 @@ public:
      * pageBreakData - at least 1 correct element (having all the chart) must be added.
      * noLabers, noOverflownLabels - if we use text reflow and want to report overflows (leaving them 0 is OK)
      * used_shapes - if shapes are used (else OLE embedding will miss them) (leaving it to empty is OK)
-     * ismapData - if requestes by the caller (else we will not generate ismaps even if the user wanted them) (leaving it to empty is OK)
+     * ismapData - if requests by the caller (else we will not generate ismaps even if the user wanted them) (leaving it to empty is OK)
      * AllElements, AllCovers - if one wishes tracking (leaving it to empty is OK)
      * file_url, file_info - if one has these chart options (leaving it to empty is OK)*/
     virtual void CompleteParse(bool autoPaginate = false,
@@ -334,7 +320,7 @@ public:
     * @param [in] dx The right margin.
     * @param [in] y The top of the label.
     * @param [in] cx If also specified, we center around it for centered lines,
-    *                but taking care not to go ouside the margings.
+    *                but taking care not to go outside the margins.
     *                If the line is wider than `dx-sx` we will go outside
     *                as little as possible (thus we center around `(sx+dx)/2`.
     * @param [in] c The center of rotation in case of a slanted arrow label.
@@ -354,7 +340,7 @@ public:
     *               In the result this is the Top/Right/Left edge of text for side==END/LEFT/RIGHT.
     * @param [in] side from which direction is the text read. For END it will be laid out horizontally.
     * @param [in] c If also specified, we center around it for centered lines,
-    *                but taking care not to go ouside the margings.
+    *                but taking care not to go outside the margins.
     *                If the line is wider than `d-s` we will go outside
     *                as little as possible (thus we center around `(s+d)/2`.*/
     void RegisterLabel(const Label &l, LabelInfo::LabelType type,
@@ -381,7 +367,7 @@ public:
     virtual void DrawHeaderFooter(Canvas &canvas, unsigned page) = 0;
     /** Draws one complete chart page or all pages.
     * We draw header, footer and all chart content.
-    * Expects the canvas to be 'prepared' and will unprepare it.
+    * Expects the canvas to be 'prepared' and will un-prepare it.
     * @param canvas The canvas to draw on
     * @param [in] pageBreaks Governs if we draw page break indicators for multi-page charts.
     * @param [in] page The page to draw. Zero means the whole chart. */
@@ -415,7 +401,7 @@ public:
  * 'Ctx' must be a descendant of ContextBase<>, 'Prog' must be a descendant of ProgressBase,
  * 'RefData' must be a descendant of RefNameData,
  * and 'Pagedata' must be a descendant of PageBreakData. By supplying your own classes
- * for these you can customize the pagination, referencing and/or style behaviour
+ * for these you can customize the pagination, referencing and/or style behavior
  * (Context is really a set of active styles).
  * This class can actually collect and hold designs, references and pagination data.*/
 template <class Ctx = ContextBase<Style>, class Prog = ProgressBase, class RefData = RefNameData, class PageData = PageBreakData>
@@ -426,11 +412,11 @@ public:
 
     /** @name Members holding chart content (entities, arcs, definitions, etc.)
     * @{ */
-    Prog                          Progress;        ///<The element tracking our progress
-    std::list<Ctx>                Contexts;        ///<A stack of ContextBase objects used during parsing
-    std::map<string, Ctx>         Designs;         ///<A set of named designs. Each one was created via a `defdesign` command, except `plain`.
-    std::map<string, RefData>     ReferenceNames;  ///<A set of named arcs. Each one was named via the 'refname' attribute.
-    std::vector<PageData>         pageBreakData;   ///<Starting xy pos and (later) other info for each page. pageBreakData[0].xy is always 0,0.
+    Prog                                   Progress;      ///<The element tracking our progress
+    std::list<Ctx>                         Contexts;      ///<A stack of ContextBase objects used during parsing
+    std::map<string, Ctx, std::less<>>     Designs;       ///<A set of named designs. Each one was created via a `defdesign` command, except `plain`.
+    std::map<string, RefData, std::less<>> ReferenceNames;///<A set of named arcs. Each one was named via the 'refname' attribute.
+    std::vector<PageData>                  pageBreakData; ///<Starting xy pos and (later) other info for each page. pageBreakData[0].xy is always 0,0.
     /** @} */
 
     virtual ProgressBase *GetProgress() override { return &Progress; }
@@ -443,9 +429,9 @@ public:
     std::map<std::string, bool> GetDesignNames() const override;
     std::string GetDesignNamesAsString(bool full) const override;
     ContextPtrCatalog ExportDesigns() const override; ///<Creates a copy of all designs
-    void ImportDesigns(const ContextPtrCatalog&) override; ///<Fills in the Designs memeber with the content of this.
+    void ImportDesigns(const ContextPtrCatalog&) override; ///<Fills in the Designs member with the content of this.
     const Context* GetCurrentContext() const override  { return Contexts.size() ? &Contexts.back() : nullptr; }
-    const RefNameData *GetReference(const std::string &refname) const override;
+    const RefNameData *GetReference(std::string_view refname) const override;
     Ctx &MyCurrentContext() { return dynamic_cast<Ctx&>(Contexts.back()); }
     const Ctx &MyCurrentContext() const { return dynamic_cast<const Ctx&>(Contexts.back()); }
     /** Push a new, fresh context to the context stack.
@@ -463,8 +449,8 @@ public:
     /** Duplicates the last context on the stack. Used when a user opens a new context via a '{'.*/
     void PushContext(const FileLineCol &l)
         { auto &last = Contexts.back(); Contexts.emplace_back(last, last.GetParseMode(), l); }
-    const Procedure *GetProcedure(const std::string &name) const override;
-    ProcParamResolved *GetParameter(const std::string &name) override;
+    const Procedure *GetProcedure(std::string_view name) const override;
+    ProcParamResolved *GetParameter(std::string_view name) override;
     void SetVariable(ProcParamDef &&p, const FileLineColRange &def);
     void SetVariable(gsl::owner<ProcParamDef*> p, const FileLineColRange &def)
         {if (p) {SetVariable(std::move(*p), def); delete p;}}
@@ -474,9 +460,9 @@ public:
     /** Sets the font in all contexts and all full designs. */
     void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange& l) override;
     /** Adds an attribute list to each of a list of styles.*/
-    void AddAttributeListToStyleList(const AttributeList &al, const std::list<std::string> &styles);
+    void AddAttributeListToStyleList(const AttributeList &al, const std::vector<std::string> &styles);
     /** Adds an attribute list to each of a list of styles.*/
-    void AddAttributeListToStyleList(gsl::owner<AttributeList *>al, const std::list<std::string> *styles)
+    void AddAttributeListToStyleList(gsl::owner<AttributeList *>al, const std::vector<std::string> *styles)
         {if (al && styles) AddAttributeListToStyleList(*al, *styles); delete al; }
     /** Draws page breaks (dotted or dashed lines) */
     virtual void DrawPageBreaks(Canvas &canvas);
@@ -538,7 +524,7 @@ void ChartBase<Ctx, Prog, RefData, PageData>::ImportDesigns(const ContextPtrCata
 }
 
 template <class Ctx, class Prog, class RefData, class PageData>
-const RefNameData *ChartBase<Ctx, Prog, RefData, PageData>::GetReference(const std::string &refname) const
+const RefNameData *ChartBase<Ctx, Prog, RefData, PageData>::GetReference(std::string_view refname) const
 {
     auto i = ReferenceNames.find(refname);
     if (i==ReferenceNames.end())
@@ -549,7 +535,7 @@ const RefNameData *ChartBase<Ctx, Prog, RefData, PageData>::GetReference(const s
 }
 
 template <class Ctx, class Prog, class RefData, class PageData>
-const Procedure *ChartBase<Ctx, Prog, RefData, PageData>::GetProcedure(const std::string & name) const
+const Procedure *ChartBase<Ctx, Prog, RefData, PageData>::GetProcedure(std::string_view name) const
 {
     for (auto i = Contexts.rbegin(); !(i==Contexts.rend()); i++) {
         auto p = i->Procedures.find(name);
@@ -560,7 +546,7 @@ const Procedure *ChartBase<Ctx, Prog, RefData, PageData>::GetProcedure(const std
 }
 
 template <class Ctx, class Prog, class RefData, class PageData>
-ProcParamResolved* ChartBase<Ctx, Prog, RefData, PageData>::GetParameter(const std::string & name)
+ProcParamResolved* ChartBase<Ctx, Prog, RefData, PageData>::GetParameter(std::string_view name)
 {
     //We accept a parameter with the searched name only if we are
     //parsing or replaying a procedure. Else only variables play.
@@ -602,7 +588,7 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::SetVariable(ProcParamDef &&
                                                   std::forward_as_tuple(std::move(p.default_value), p.linenum_default_value, false));
         }
     } else
-        Error.Error(def.end.NextChar(), "Missing value.");
+        Error.Error(def.end, "Missing value.");
 }
 
 template<class Ctx, class Prog, class RefData, class PageData>
@@ -620,7 +606,7 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::PopContext()
 
 template<class Ctx, class Prog, class RefData, class PageData>
 inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::string& face, const std::string& lang,
-                                                                    const FileLineColRange& l) {
+                                                                    const FileLineColRange&) {
     //We need to set the font in the current context, but also in all full designs
     //(even if setting it) - so that if the design is used later, we have this font
     if (face.length()) {
@@ -641,13 +627,13 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::s
 
 template <class Ctx, class Prog, class RefData, class PageData>
 void ChartBase<Ctx, Prog, RefData, PageData>::AddAttributeListToStyleList(
-        const AttributeList &al, const std::list<std::string> &sl) {
+        const AttributeList &al, const std::vector<std::string> &sl) {
     if (al.size())
         for (auto &a : al) {
-            std::list<string> problem;
+            std::vector<string> problem;
             bool had_generic = false;
             for (auto &s : sl) {
-                auto style = MyCurrentContext().styles.GetStyle(s); //may be default style
+                auto style = MyCurrentContext().styles.GetStyle(s); //may be the default style
                 if (style.write().AddAttribute(*a, this))
                     MyCurrentContext().styles[s] = style;
                 else {
@@ -676,9 +662,9 @@ void ChartBase<Ctx, Prog, RefData, PageData>::AddAttributeListToStyleList(
             }
             Error.Error(*a, false, msg);
         }
-    else //empty styles mey be defined
+    else //empty styles may be defined
         for (auto &s : sl)
-            MyCurrentContext().styles[s] = MyCurrentContext().styles.GetStyle(s); //may be default style
+            MyCurrentContext().styles[s] = MyCurrentContext().styles.GetStyle(s); //may be the default style
 }
 
 /** Draw the page breaks */
@@ -730,7 +716,7 @@ template <class Ctx, class Prog, class RefData, class PageData>
 void ChartBase<Ctx, Prog, RefData, PageData>::DrawHeaderFooter(Canvas &canvas, unsigned page)
 {
     _ASSERT(page<=pageBreakData.size());
-    canvas.PrepareForHeaderFoorter();
+    canvas.PrepareForHeaderFooter();
 
     if (copyrightTextHeight==0 || copyrightText.empty()) return; //Dont do nothing if there is no copyright text.
 
@@ -766,6 +752,20 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::SetToEmpty()
     Chart::SetToEmpty();
 }
 
+/** The state parsing may reach an opening '{' brace.*/
+enum class EScopeOpenMode
+{
+    NORMAL,      ///<Nothing special (Used also by procedure definitions. There the fact that we parse a procedure can be captured by bison rules.)
+    PROC_REPLAY  ///<A procedure invocation preceded the brace, which is then from the text of the proc body
+};
 
+/* For languages supporting procedures and replay,
+ * this structure is used during parsing between the
+ * header of the procedure and its opening '{'. */
+struct procedure_parse_helper {
+    ParameterContext last_procedure_params;    ///<Place parameter list of a procedure definition header here to be available to the body
+    const Procedure* last_procedure = nullptr; ///<After parsing a procedure header put the looked-up procedure here.
+    EScopeOpenMode   open_context_mode = EScopeOpenMode::NORMAL; ///<What was there before a scope opening?
+};
 
 #endif
\ No newline at end of file
