@@ -1,6 +1,6 @@
 /*
 This file is part of Msc-generator.
-Copyright (C) 2008-2022 Zoltan Turanyi
+Copyright (C) 2008-2023 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -33,7 +33,7 @@ along with Msc-generator.  If not, see <http://www.gnu.org/licenses/>.
 std::string VersionText(char a=LIBMSCGEN_MAJOR, char b = LIBMSCGEN_MINOR, char c = LIBMSCGEN_SUPERMINOR);
 std::string ReadFile(FILE *in, bool till_zero=false);
 std::string version(const LanguageCollection &languages);
-std::string licence(const LanguageCollection &languages);
+std::string license(const LanguageCollection &languages);
 
 using LoadDataSet = std::map<std::string, std::string>;
 /** Unpack the load data string by language type.*/
@@ -65,7 +65,7 @@ class Examples {
 public:
     explicit Examples(const LanguageCollection* languages) noexcept : L(languages) {}
     /** Examine each of the example dirs and load the examples. If we find any examples
-     * in a dir, we skip checking the rest of the dirs. A valid example is a file with a 
+     * in a dir, we skip checking the rest of the dirs. A valid example is a file with a
      * chart text extension, where the first 3 lines start with a double hash-mark (##).*/
     std::pair<int, std::string> Load(const std::vector<std::string>&example_dirs, OpenNamedFileFunction *file_open_proc);
     /** Returns true of we have loaded at least one example for this language. */
@@ -78,8 +78,7 @@ public:
 
 
 struct GUIInit {
-    LanguageCollection &languages;
-    std::vector<std::pair<std::string, std::string>> &design_files;
+    LanguageCollection languages;
     Chart::FileReadProcedure *file_read_proc = nullptr;
     void *file_read_proc_param = nullptr;
     OpenNamedFileFunction *file_open_proc = nullptr;
@@ -93,8 +92,8 @@ struct GUIInit {
 
 std::variant<int, GUIInit>
 do_main(const std::vector<std::string> &args,
-        LanguageCollection &languages,
-        std::vector<std::pair<std::string, std::string>> &design_files,
+        LanguageCollection &&languages,
+        std::vector<std::pair<std::string, std::string>> &&design_files,
         OpenNamedFileFunction ReadNamedFile,
         Chart::FileReadProcedure file_read_proc, void *file_read_proc_param,
         std::string csh_textformat,
@@ -102,4 +101,17 @@ do_main(const std::vector<std::string> &args,
         ProgressBase::ProgressCallback cb = nullptr, void *param = nullptr,
         std::string *load_data = nullptr);
 
+/** Returns the text description of a reference, e.g., "Doxygen reference",
+ * "Ctrl+Click to open in browser" or just "Reference" to be displayed in a
+ * tooltip. If we also return true, the link can be given to a browser.*/
+inline std::pair<std::string_view, bool> EvaluateReferenceForGUI(std::string_view ref) noexcept {
+    if (CaseInsensitiveBeginsWith(ref, "\\ref "))
+        return {"Doxygen reference", false};
+    if (CaseInsensitiveBeginsWith(ref, "http://") ||
+        CaseInsensitiveBeginsWith(ref, "https://") ||
+        CaseInsensitiveBeginsWith(ref, "ftp://"))
+        return {"Ctrl+Click to open in browser", true};
+    return {"Reference", false};
+}
+
 #endif /* COMMANDLINE_H */
