@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -418,6 +418,10 @@ struct Range {
         if (from>=till) return 0;
         return (v-from)/Spans();
     }
+    constexpr double operator[](unsigned i) const noexcept { return i ? till : from; }
+    constexpr double operator[](bool i) const noexcept { return i ? till : from; }
+    constexpr double& operator[](unsigned i) noexcept { return i ? till : from; }
+    constexpr double& operator[](bool i) noexcept { return i ? till : from; }
 };
 
 /** A basic rectangle structure.
