@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include <algorithm>
 #include <memory>
 #include <compare>
+#include <cstring>
 
 #ifndef _ASSERT
 #define  _ASSERT(A) assert(A)
@@ -39,6 +40,39 @@
 #define NDEBUG
 #endif
 
+inline std::string my_strerror(int errno_) {
+    char buff[512] = {'\0'};
+    //strerror() is not thread safe and results in all kinds of
+    //compiler warnings. There are safe variants, but...
+#ifdef _WIN32
+    //Well, windows has strerror_s(), which is C11 standard
+    strerror_s(buff, sizeof(buff), errno_);
+#else
+    //POSIX and GCC each have strerror_r(), but what a mess: https://linux.die.net/man/3/strerror_r
+    //The precondition macros listed there do not work on MacOS, so we do this instead.
+    using strerror_r_ret_type = decltype(strerror_r(errno_, buff, sizeof(buff)));
+    constexpr bool strerror_r_rets_int = std::is_same_v<strerror_r_ret_type, int>;
+    constexpr bool strerror_r_rets_charp = std::is_same_v<strerror_r_ret_type, char *>;
+    //Note: "Outside a template, a discarded statement is fully checked. if constexpr is not a
+    //substitute for the #if preprocessing directive" So both branches must be valid C++.
+    if constexpr (strerror_r_rets_int) {
+        if (strerror_r(errno_, buff, sizeof(buff)))
+            snprintf(buff, sizeof(buff), "Could not get error string for errno=%d", errno_);
+    } else if constexpr (strerror_r_rets_charp) {
+        char* b = (char*)strerror_r(errno_, buff, sizeof(buff));
+        if (b!=buff) {
+            strncpy(buff, b, sizeof(buff)-1);
+            buff[sizeof(buff)-1] = 0;
+        }
+    } else
+        static_assert(strerror_r_rets_charp || strerror_r_rets_int, "strerror_r() returns neither char* nor int.");
+#endif
+    return buff;
+}
+
+template<typename T, typename... U> requires (std::equality_comparable_with<T, U> && ...)
+bool is_any_of(T t, U... u) { return ((t == u) || ...); }
+
 class FileLineCol;
 
 /** Contains a distance in the input file between two positions.
@@ -124,6 +158,7 @@ public:
     }
     bool operator ==(const FileLineCol& a) const noexcept { return std::is_eq(operator<=>(a)); }
     FileLineCol& operator = (const FileLineCol& o) noexcept = default;
+    FileLineCol& operator = (FileLineCol&& o) noexcept = default;
     FileLineCol NextChar() const noexcept { return AdvanceCol(1); }
     FileLineCol AdvanceCol(unsigned u) const noexcept { FileLineCol tmp(*this); tmp.col += u; return tmp; }
     FileLineCol AdvanceUTF8(std::string_view text) const noexcept;
@@ -137,7 +172,8 @@ struct FileLineColRange {
     FileLineCol start; ///< The first character of the range
     FileLineCol end;   ///< The last character of the range. Equals `start` for single character ranges.
     FileLineColRange TopOfStack() const noexcept { return {start.TopOfStack(), end.TopOfStack()}; }
-    FileLineColRange &IncStartCol(size_t i=1) noexcept {start.col+=i; return *this;}
+    FileLineColRange& IncStartCol(size_t i = 1) &  noexcept { start.col += i; return *this; }
+    FileLineColRange&&IncStartCol(size_t i = 1) && noexcept { start.col += i; return std::move(*this);}
     void MakeInvalid() noexcept {start.MakeInvalid(); end.MakeInvalid();}
     constexpr bool IsInvalid() const noexcept {return start.IsInvalid() || end.IsInvalid();}
     void Push(const FileLineCol &o, EInclusionReason r) { start.Push(o, r); end.Push(o, r); }
@@ -147,10 +183,16 @@ struct FileLineColRange {
     FileLineColRange CreatePushed(const FileLineColRange &o, EInclusionReason r) const
         { return {start.CreatePushed(o.start, r), end.CreatePushed(o.end, r)}; }
     std::strong_ordering operator<=>(const FileLineColRange &o) const noexcept = default;
-    FileLineColRange &operator +=(const FileLineColRange &o) noexcept { start = std::min(start, o.start); end = std::min(end, o.end); return *this; }
+    FileLineColRange &operator +=(const FileLineColRange &o) & noexcept { start = std::min(start, o.start); end = std::min(end, o.end); return *this; }
+    FileLineColRange&&operator +=(const FileLineColRange &o) &&noexcept { start = std::min(start, o.start); end = std::min(end, o.end); return std::move(*this); }
+    friend FileLineColRange operator +(const FileLineColRange& a, const FileLineColRange& b) noexcept
+        { return FileLineColRange{std::min(a.start, b.start),std::max(a.end, b.end)}; }
+    friend FileLineColRange operator +(FileLineColRange&& a, FileLineColRange&& b) noexcept
+        { return FileLineColRange{std::move(std::min(a.start, b.start)), std::move(std::max(a.end, b.end))}; }
     std::string Print() const { return start.Print() + end.Print(); }
     /** Read a location range from the string. On success, we step 's' to the next char after & return true, else we leave us unchanged.*/
     bool Read(std::string_view &s) { FileLineColRange r;  if (r.start.Read(s) && r.end.Read(s)) { *this = r; return true; } return false; }
+    FileLineCol After() const noexcept { return end.NextChar(); }
 };
 
 /** Returns which of two ranges are shorter.*/
@@ -185,7 +227,7 @@ struct ErrorElement
     FileLineCol relevant_line; ///<The location to which the information applies. This is shown.
     FileLineCol ordering_line; ///<The location used at ordering the errors. Different for auxiliary info.
     EType type;                ///<Error or warning or technical info
-    ELineType line_type;       ///<Wether this is the main diagnostic message or supplementary info
+    ELineType line_type;       ///<Whether this is the main diagnostic message or supplementary info
     bool isOnlyOnce;           ///<True if messages with this text shall only be displayed once.
     std::string text;          ///<The full text of the error/warning including location, etc.
     std::string message;       ///<The original text of the error/warning.
@@ -205,8 +247,8 @@ protected:
     bool unique_push_back(const ErrorElement &e) { if (IsInStore(e)) return false;  Messages.push_back(e); return true; } ///<Push back an element, but only if one with similar text does not exist.
     bool unique_push_back(ErrorElement &&e) { if (IsInStore(e)) return false;  Messages.push_back(std::move(e)); return true; } ///<Push back an element, but only if one with similar text does not exist.
 
-    void Add(FileLineCol linenum, FileLineCol linenum_ord, std::string_view s, std::string_view once, ErrorElement::EType t, bool omit_error_warning, ErrorElement::ELineType lt = ErrorElement::ELineType::Main);
-    bool AddOne(FileLineCol linenum, FileLineCol linenum_ord, std::string_view s, ErrorElement::EType t, bool is_once, bool add_parenthesis, bool omit_error_warning, ErrorElement::ELineType lt);
+    void Add(const FileLineCol &linenum, const FileLineCol& linenum_ord, std::string_view s, std::string_view once, ErrorElement::EType t, bool omit_error_warning, ErrorElement::ELineType lt = ErrorElement::ELineType::Main);
+    bool AddOne(const FileLineCol& linenum, const FileLineCol& linenum_ord, std::string_view s, ErrorElement::EType t, bool is_once, bool add_parenthesis, bool omit_error_warning, ErrorElement::ELineType lt);
     void Add(const Attribute &a, bool atValue, std::string_view s, std::string_view once, ErrorElement::EType t);
 
 public:
@@ -276,10 +318,17 @@ public:
     * @param [in] name The name of the attribute to use instead*/
     void WarnMscgenAttr(const Attribute &a, bool isOption, std::string_view name);
 
+    void DeleteLastErrorAt(FileLineCol linenum) {
+        while (Messages.size()
+               && Messages.back().type == ErrorElement::EType::Error
+               && Messages.back().ordering_line == linenum)
+            Messages.pop_back();
+    }
+
     /** Print all the errors (and warnings) onto a string*/
     std::string Print(bool oWarnings, bool oTechnicalInfo) const;
     /** True if there were errors added*/
-    bool hasErrors() const { return Messages.end()!=std::find_if(Messages.begin(), Messages.end(), [](auto &m) {return m.type==ErrorElement::EType::Error; }); }
+    bool hasErrors() const { return Messages.end()!=std::ranges::find(Messages, ErrorElement::EType::Error, &ErrorElement::type); }
     /** True if there were fatal errors added*/
     bool hasFatal() const {return hadFatal;}
     /** Sort errors and warnings by location */
@@ -311,12 +360,12 @@ public:
         auto p = GetError(num, oWarnings, oTechnicalInfo);
         return p ? p->message : std::string_view{};
     }
-    ErrorElement FormulateElement(FileLineCol linenum, FileLineCol linenum_ord, ErrorElement::EType t,
+    ErrorElement FormulateElement(const FileLineCol& linenum, const FileLineCol& linenum_ord, ErrorElement::EType t,
                                   bool is_once, bool add_para, bool omit_message_type, ErrorElement::ELineType lt,
                                   std::string_view msg) const;
     /** Clearing all collected errors and warnings, but keeps the files.*/
     void Clear() noexcept {Messages.clear(); hadFatal=warn_mscgen=false; }
-    /** Copies both files and messages from anoter Error collection.
+    /** Copies both files and messages from another Error collection.
      * It is best if files do not overlap. Useful only when merging errors
      * from different file parses.*/
     MscError &operator+=(const MscError &);
@@ -328,7 +377,7 @@ public:
  * Used by both Msc and Csh classes.*/
 enum class EMscgenCompat
 {
-    NO_COMPAT=0,   ///<Never switch to compatibilit mode
+    NO_COMPAT=0,   ///<Never switch to compatibility mode
     AUTODETECT=1,  ///<Switch to compatibility mode if 'msc {...}' detected
     FORCE_MSCGEN=2 ///<Start in mscgen compatibility mode eve if no 'msc {...}' detected
 };
