@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -29,8 +29,8 @@
 #include "gvplugin.h"
 #include "gvplugin_render.h"
 #include "gvplugin_device.h"
-#include "parse_tools.h" //for remove_head_tail_whitespace()
-#include "chartbase.h" 
+#include "parser_tools.h" //for remove_head_tail_whitespace()
+#include "chartbase.h"
 #include "gvgraphs.h"
 
 #ifdef _WIN32
@@ -123,17 +123,16 @@ void GraphDrawingPrimitive::Draw(Canvas &canvas, const ShapeCollection &shapes)
             //Dont stroke if of same color as a solid fill
             const LineAttr l = fill.gradient!=EIntGradType::NONE || fill.color!=line.color
                 ? line : LineAttr::None();
-            const Block bb = label.IsEmpty() 
+            const Block bb = label.IsEmpty()
                 ? Block(false) : Block(sx, dx, y, y+label.getTextWidthHeight().y);
             canvas.Add(GSShape(contour, l, fill, shadow, bb, label));
         }
     }
 }
 
-
 bool GraphElement::AddAttribute(const Attribute &a, Chart *chart)
-{ 
-    return style.write().AddAttribute(a, chart); 
+{
+    return style.write().AddAttribute(a, chart);
 }
 
 void GraphElement::AddAttributeList(const GraphStyle *s, Chart *chart)
@@ -142,20 +141,23 @@ void GraphElement::AddAttributeList(const GraphStyle *s, Chart *chart)
     style.write() += *s;
     if (!style.read().attributes.contains("label") ||
         !style.read().attributes.contains("original_label")) return;
-    
+
     //if the element has "label" and "original_label" attributes, we adjust them.
     auto label_w = style.write().attributes.find("label");
     StringFormat::ExpandReferences(label_w->second.value, chart, label_w->second.linenum_value, nullptr,
-                                    false, true, StringFormat::LABEL, true);
+                                   false, true, StringFormat::LABEL, true);
     StringFormat basic;
     basic.Default();
     auto i = style.read().attributes.find("label_format");
     if (i!=style.read().attributes.end())
         basic.Apply(i->second.value.c_str()); //do not change 'attributes["label_format"].value'
+    i = style.read().attributes.find("link_format");
+    if (i != style.read().attributes.end())
+        basic.AddAttribute(Attribute(i->first, i->second.value, {i->second.linenum_attr, i->second.linenum_attr}, {i->second.linenum_value, i->second.linenum_value}, true), chart, EStyleType::ELEMENT);
     //In this apply we would not recognize color or style names, hence we expanded them above
     basic.Apply(label_w->second.value.c_str()); //do not change 'attributes["label"].value'
     //Now expand the \c() and \s(), too, based on the basic formatting
-    StringFormat::ExpandReferences(label_w->second.value, nullptr, label_w->second.linenum_value, &basic,
+    StringFormat::ExpandReferences(label_w->second.value, chart, label_w->second.linenum_value, &basic,
                                     false, true, StringFormat::LABEL, true);
     auto orig_w = style.write().attributes.find("original_label");
     orig_w->second.value = label_w->second.value;
@@ -178,7 +180,7 @@ void GraphElement::AddDrawingPrimitive(Path && p, const LineAttr & l)
     else
         drawingList.emplace_back(std::move(p), l);
 }
-             
+
 /** Fills in a 'shadow' attribute and reports errors if needed */
 ShadowAttr GraphElement::GetShadow(Chart* chart) const
 {
@@ -197,7 +199,7 @@ ShadowAttr GraphElement::GetShadow(Chart* chart) const
     return shadow;
 }
 
-/** Insert our attributes into cgraph. 
+/** Insert our attributes into cgraph.
  * @param in_gv This is our object in cgraph (already added by AddToGvGraph() of a descendant.
  * We take care to convert 'double' and 'triple' line styles to ones understandable to graphviz.*/
 void GraphElement::AddAttributesToGraph(void * in_gv) const
@@ -207,7 +209,7 @@ void GraphElement::AddAttributesToGraph(void * in_gv) const
             continue; //skip those that are not interpreted by graphviz
         else if (p.first=="style") {
             ParsedStyle ps;
-            ps.Parse(p.second.value.c_str());
+            ps.Parse(p.second.value);
             if ((ps.line=="double" || ps.line=="triple"))
                 ps.line = "bold";
             std::string style = ps;
@@ -228,10 +230,10 @@ void GraphElement::ShiftBy(const XY &xy)
 
 void GraphElement::PostPosProcess(Canvas &canvas, Chart * ch)
 {
-    //add a shadow draw element if needed    
-    if (ShadowAttr shadow = GetShadow(ch); !shadow.IsEmpty()) {
+    //add a shadow draw element if needed
+    if (ShadowAttr shadow = GetShadow(ch); !shadow.IsNone()) {
         //find the largest fill contour
-        const auto max = std::ranges::max_element(drawingList, {}, [](const GraphDrawingPrimitive& d) 
+        const auto max = std::ranges::max_element(drawingList, {}, [](const GraphDrawingPrimitive& d)
                                                   { return d.contour.IsEmpty() ? 0 : d.contour.GetArea(); });
         if (max!=drawingList.end()) {
             max->shadow = shadow;
@@ -249,7 +251,7 @@ void GraphElement::PostPosProcess(Canvas &canvas, Chart * ch)
                 ; l.path.IsEmpty() && l.contour.IsEmpty() && !l.label.IsEmpty()) {
                 //Merge them
                 std::swap(c.label, l.label);
-                c.sx = l.sx; 
+                c.sx = l.sx;
                 c.dx = l.dx;
                 c.y = l.y;
                 drawingList.pop_back();
@@ -319,11 +321,11 @@ int Graph::DotError(char *msg)
             else if (2==sscanf(s.c_str(), "node %79s port %79s unrecognized", n, p)) {
                 //chop terminating ',' from node
                 if (n[strlen(n)-1]==',')
-                    n[strlen(n)-1] = 0; 
+                    n[strlen(n)-1] = 0;
                 //search for an edge with this node/port combo
                 //first see if we have such a node
                 GraphNode *node = Graph::_current->GetNode(n);
-                if (node) 
+                if (node)
                     //if so, see if we have an edge with this port
                     for (const auto &e : Graph::_current->edges) {
                         if (e->n1 == node && strncmp(p, e->port1.c_str(), strlen(p))==0)
@@ -346,14 +348,14 @@ int Graph::DotError(char *msg)
 
 
 /** Adds the attributes to the list of attribute names to bind.
- * graphviz requires the registration (binding) of all attribute 
+ * graphviz requires the registration (binding) of all attribute
  * names (globally) before used for a given kind of object
  * (graph, edge or node).
  * @param t [in] Specifies what kind of object the attributes are for.
  * @param s [in] The list of attribute names to add.*/
 void Graph::AddAttrName(GraphStyle::EGraphElementType t, const GraphStyle & s)
 {
-    if (t==GraphStyle::EGraphElementType::ANY) 
+    if (t==GraphStyle::EGraphElementType::ANY)
         for (auto &a : s.attributes) {
             attr_node.insert(a.first);
             attr_edge.insert(a.first);
@@ -370,7 +372,7 @@ void Graph::AddAttrName(GraphStyle::EGraphElementType t, const GraphStyle & s)
 /** Searches for a node in the graph by name. Returns nullptr if not found.
  * Only the nodes currently part of the graph are searched.
  * (The ones not used but still part of storage_nodes are not.)*/
-GraphNode *Graph::GetNode(const std::string &name) const
+GraphNode *Graph::GetNode(std::string_view name) const
 {
     for (auto &p: nodes)
         if (p->name==name)
@@ -383,28 +385,10 @@ GraphNode *Graph::GetNode(const std::string &name) const
     return nullptr;
 }
 
-/** Searches for a node in the graph by name. Returns nullptr if not found.
- * Only the nodes currently part of the graph are searched.
- * (The ones not used but still part of storage_nodes are not.)*/
-GraphNode *Graph::GetNode(const char *name) const
-{
-    if (name==nullptr) return nullptr;
-    for (auto &p: nodes)
-        if (p->name==name)
-            return p;
-    for (auto &s: subgraphs) {
-        auto ret = s->GetNode(name);
-        if (ret)
-            return ret;
-    }
-    return nullptr;
-}
-
-
 /** Searches for a subgraph in the graph by name. Returns nullptr if not found.
  * Only the subgraphs currently part of the graph are searched.
  * (The ones not used but still part of storage_subgraphs are not.)*/
-SubGraph *Graph::GetSubGraph(const std::string & n) const
+SubGraph *Graph::GetSubGraph(std::string_view n) const
 {
     for (auto &s: subgraphs) {
         SubGraph *ret = s->GetSubGraph(n);
@@ -412,19 +396,6 @@ SubGraph *Graph::GetSubGraph(const std::string & n) const
     }
     return nullptr;
 }
-
-/** Searches for a subgraph in the graph by name. Returns nullptr if not found.
- * Only the subgraphs currently part of the graph are searched.
- * (The ones not used but still part of storage_subgraphs are not.)*/
-SubGraph *Graph::GetSubGraph(const char *n) const
-{
-    if (n==nullptr) return nullptr;
-    for (auto &s: subgraphs) {
-        SubGraph *ret = s->GetSubGraph(n);
-        if (ret) return ret;
-    }
-    return nullptr;
-}
 /** Returns the first edge found between these nodes. */
 GraphEdge *Graph::GetFirstEdge(const GraphNode * _1, const GraphNode *_2) const
 {
@@ -565,6 +536,17 @@ void Graph::PseudoRenderFrom(Chart*chart)
         boundingBox.x.from = boundingBox.x.till = boundingBox.y.from = boundingBox.y.till = 0;
 }
 
+void Graph::GraphCollectIsMapElements(Canvas& canvas, Chart& chart) const {
+    GraphElement::GraphCollectIsMapElements(canvas, chart);
+    for (auto& pSubGraph : storage_subgraphs)
+        pSubGraph->GraphCollectIsMapElements(canvas, chart);
+    for (auto& pNode : storage_nodes)
+        pNode->GraphCollectIsMapElements(canvas, chart);
+    for (auto& pEdge : edges)
+        pEdge->GraphCollectIsMapElements(canvas, chart);
+}
+
+
 /** Draws the graph based on the drawing primitives
  *  collected in PseudoRenderFrom(). */
 void Graph::Draw(Canvas &canvas, const ShapeCollection &shapes, EGraphDrawPass p) const
@@ -582,10 +564,10 @@ void Graph::Draw(Canvas &canvas, const ShapeCollection &shapes, EGraphDrawPass p
 /** Correct deletion order, albeit no longer needed.*/
 Graph::~Graph()
 {
-    edges.clear(); 
-    subgraphs.clear(); 
-    nodes.clear(); 
-    name.clear(); 
+    edges.clear();
+    subgraphs.clear();
+    nodes.clear();
+    name.clear();
     attr_node.clear();
     attr_edge.clear();
     attr_graph.clear();
@@ -665,7 +647,7 @@ void Graph::AdjustBkArea(const Range &total_x) noexcept {
 }
 
 std::optional<BackgroundFill> Graph::GetBkFill() const noexcept {
-    if (drawingList.size()==1 && drawingList.front().contour.IsRectangle()) 
+    if (drawingList.size()==1 && drawingList.front().contour.IsRectangle())
         return BackgroundFill{drawingList.front().fill, drawingList.front().contour.GetBoundingBox()};
     return {};
 }
@@ -673,8 +655,8 @@ std::optional<BackgroundFill> Graph::GetBkFill() const noexcept {
 
 void SubGraph::AddToGvGraph() const
 {
-    in_gv = agsubg(parent ? parent->GetGv() : graph.GetGv(), 
-                   name.length() ? name.c_str() : nullptr, 
+    in_gv = agsubg(parent ? parent->GetGv() : graph.GetGv(),
+                   name.length() ? name.c_str() : nullptr,
                    /*createflag==*/ 1);
     AddAttributesToGraph(in_gv);
     //add subgraphs
@@ -743,7 +725,7 @@ bool SubGraph::RemoveEmptySubGraphs()
 }
 
 /** Searches for a node in the subgraph by name. Returns nullptr if not found.*/
-GraphNode *SubGraph::GetNode(const std::string &name) const
+GraphNode *SubGraph::GetNode(std::string_view name) const
 {
     for (auto &p: nodes)
         if (p->name==name)
@@ -756,24 +738,9 @@ GraphNode *SubGraph::GetNode(const std::string &name) const
     return nullptr;
 }
 
-/** Searches for a node in the subgraph by name. Returns nullptr if not found.*/
-GraphNode *SubGraph::GetNode(const char *name) const
-{
-    if (name==nullptr) return nullptr;
-    for (auto &p: nodes)
-        if (p->name==name)
-            return p;
-    for (auto &s: subgraphs) {
-        auto ret = s->GetNode(name);
-        if (ret)
-            return ret;
-    }
-    return nullptr;
-}
-
 
 /** Searches for a subgraph in the subgraph by name. Returns nullptr if not found.*/
-SubGraph *SubGraph::GetSubGraph(const std::string & n)
+SubGraph *SubGraph::GetSubGraph(std::string_view n)
 {
     if (n==name) return this;
     for (auto &s: subgraphs) {
@@ -783,19 +750,7 @@ SubGraph *SubGraph::GetSubGraph(const std::string & n)
     return nullptr;
 }
 
-/** Searches for a subgraph in the subgraph by name. Returns nullptr if not found.*/
-SubGraph *SubGraph::GetSubGraph(const char *n)
-{
-    if (n==nullptr) return nullptr;
-    if (n==name) return this;
-    for (auto &s: subgraphs) {
-        SubGraph *ret = s->GetSubGraph(n);
-        if (ret) return ret;
-    }
-    return nullptr;
-}
-
-/** Searches for a specific element in the subgraph by its pointer in graphviz. 
+/** Searches for a specific element in the subgraph by its pointer in graphviz.
  * Returns nullptr if not found.*/
 GraphElement * SubGraph::GetElement(void * in_gv_loc)
 {
@@ -836,7 +791,7 @@ void SubGraph::ShiftBy(const XY &xy)
 }
 
 
-/** Returns a list of nodes (no ports). 
+/** Returns a list of nodes (no ports).
  * Used during parsing to generate edges for constructs, like
  * "{ a b c } -> { d e f }"*/
 std::unique_ptr<GraphNodePortList> SubGraph::CopyOfNodes() const
@@ -851,7 +806,7 @@ std::unique_ptr<GraphNodePortList> SubGraph::CopyOfNodes() const
 
 void SubGraph::PostPosProcess(Canvas &canvas, Chart *ch)
 {
-    GraphElement::PostPosProcess(canvas, ch); 
+    GraphElement::PostPosProcess(canvas, ch);
     //create frame
     area_draw = area.CreateExpand(ch->trackFrameWidth) - area;
     draw_is_different = true;
@@ -879,7 +834,7 @@ void GraphNode::AddToGvGraph(const SubGraph *parent) const
 }
 
 /** Sets the label of the node to its name.
- * If the name contains no hidden escapes, we leave the label untouched (empty probably, which 
+ * If the name contains no hidden escapes, we leave the label untouched (empty probably, which
  * will be assumed to be "\N" by graphviz and be substituted by node name. If the name has
  * hidden escapes (the name had a $$ in it), we remove them and use that as a label.*/
 void GraphNode::SetLabelFromName()
@@ -889,7 +844,7 @@ void GraphNode::SetLabelFromName()
         AddAttribute(Attribute("label", l, file_pos, file_pos), nullptr); //ugly, but we assume no errors, so we skip chart
 }
 
-void GraphNode::Draw(Canvas & canvas, const ShapeCollection &shapes, EGraphDrawPass p) const 
+void GraphNode::Draw(Canvas & canvas, const ShapeCollection &shapes, EGraphDrawPass p) const
 {
     if (p!=EGraphDrawPass::NODE) return;
     DrawAll(canvas, shapes);
@@ -947,7 +902,7 @@ inline LineAttr GetLineAttr(const obj_state_t *obj)
     }
     return line;
 }
-                                               
+
 /** Copies the graphviz object's fill attributes into 'fill' */
 inline FillAttr GetFillAttr(const obj_state_t *obj)
 {
@@ -977,20 +932,20 @@ void canvas_cover_textspan(bool draw, GVJ_t *job, pointf p, textspan_t * span)
     //see if the object we take the label for, has text formatting attributes
     if (!draw) {
         Graph *gc = dynamic_cast<Graph*>(Graph::_current);
-        if (gc) 
+        if (gc)
             e = gc->GetElement(job->obj->u.n);
     }
     StringFormat sf;
     sf.Default();
 
-    //apply font type/size and identation
+    //apply font type/size and indentation
     char buff[1000];
     snprintf(buff, sizeof(buff), "\\f(%s)\\mn(%f)\\ms(%f)%s",
         span->font->name, span->font->size, span->font->size*0.8,
         span->just == 'n' ? "\\pc" : span->just=='l' ? "\\pl" : span->just=='r' ? "\\pr" : "");
     sf.Apply(buff);
     //color is in pencolor not in span->font->color
-    sf.SetColor(ColorType(job->obj->pencolor.u.rgba[0], job->obj->pencolor.u.rgba[1], 
+    sf.SetColor(ColorType(job->obj->pencolor.u.rgba[0], job->obj->pencolor.u.rgba[1],
                           job->obj->pencolor.u.rgba[2], job->obj->pencolor.u.rgba[3]));
     if (span->font->flags & HTML_BF)
         sf.Apply("\\B");
@@ -1011,9 +966,9 @@ void canvas_cover_textspan(bool draw, GVJ_t *job, pointf p, textspan_t * span)
         //TODO: dont have overline
     }
 #endif
-    char *copy = strdup(span->str);
-    std::string text = remove_head_tail_whitespace(copy);
-    free(copy);
+    std::string_view s(span->str);
+    remove_head_tail_whitespace(s);
+    std::string text(s);
     //replace "text" to the original "decorated" value
     //For labels specified via the ':: syntax' we have the original value stored with position escapes
     //Since graphviz generates a new textspan for each line, we need to search which line this is.
@@ -1076,7 +1031,7 @@ void canvas_cover_polygon(bool draw, GVJ_t * job, pointf * A, int n, int filled)
     LineAttr line = GetLineAttr(job->obj);
     FillAttr fill = GetFillAttr(job->obj);
     if (draw) {
-        if (filled)// && job->obj->fill != FILL_NONE) 
+        if (filled)// && job->obj->fill != FILL_NONE)
             Graph::_current->_canvas->Fill(c, fill);
         Graph::_current->_canvas->Line(c, line);
     } else {
@@ -1159,7 +1114,7 @@ void canvas_cover_ellipse(bool draw, GVJ_t * job, pointf * A, int filled)
     LineAttr line = GetLineAttr(job->obj);
     FillAttr fill = GetFillAttr(job->obj);
     if (draw) {
-        if (filled)// && job->obj->fill != FILL_NONE) 
+        if (filled)// && job->obj->fill != FILL_NONE)
             Graph::_current->_canvas->Fill(c, fill);
         Graph::_current->_canvas->Line(c, line);
     } else {
