@@ -148,7 +148,7 @@ public:
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     /** Draw the list of drawing primitives we have received.*/
     void DrawAll(Canvas &canvas, const ShapeCollection &shapes) const { for (auto &p: drawingList) p.Draw(canvas, shapes); }
-    void CollectIsMapElements(Canvas& canvas) final { _ASSERT(0); /*Call GraphCollectIsMapElements instead */ }
+    void CollectIsMapElements(Canvas&) final { _ASSERT(0); /*Call GraphCollectIsMapElements instead */ }
     virtual void GraphCollectIsMapElements(Canvas& canvas, Chart &chart) const { for (auto& p : drawingList) p.CollectIsMapElements(canvas, chart); }
     /** Draw this graph element (parts with this pass) */
     virtual void Draw(Canvas &canvas, const ShapeCollection &shapes, EGraphDrawPass) const = 0;
