@@ -137,21 +137,16 @@ void Entity::AddChildrenList(const EntityAppList *children, MscChart *chart)
     if (!children || children->size()==0) return;
     double min_pos = MaxVal(min_pos);
     for (auto i=children->begin(); i!=children->end(); i++) {
-        EntityRef ei = chart->AllEntities.Find_by_Name((*i)->name);
+        _ASSERT((*i)->defining);
+        const EntityRef ei = chart->AllEntities.Find_by_Name((*i)->name);
         _ASSERT(ei);
-        if (!(*i)->defining) {
-            chart->Error.Error((*i)->file_pos.start, "Cannot make an already existing entity part of a group.",
-                               "Entity remains outside this group.");
-            chart->Error.Error(ei->file_pos, (*i)->file_pos.start, "Entity '" + ei->name + "' was defined here.");
-        } else {
-            //if parent is already set this is a grandchilren
-            if (ei->parent_name.length()==0) {
-                children_names.insert((*i)->name);
-                ei->parent_name = name;
-            }
-            //search for leftmost even among grandchildren
-            min_pos = std::min(min_pos, ei->pos);
+        //if parent is already set this is a grandchildren
+        if (ei->parent_name.length()==0) {
+            children_names.insert((*i)->name);
+            ei->parent_name = name;
         }
+        //search for leftmost even among grandchildren
+        min_pos = std::min(min_pos, ei->pos);
     }
     if (collapsed) {
         for (auto i=children->begin(); i!=children->end(); i++) {
@@ -294,14 +289,15 @@ bool EntityApp::AddAttribute(const Attribute& a)
             pos = {0, a.linenum_attr.start};
             return true;
         }
-        if (!a.CheckType(EAttrType::NUMBER, chart->Error)) return true;
-        pos = {a.number, a.linenum_attr.start};
+        if (a.CheckType(EAttrType::NUMBER, chart->Error))
+            pos = {a.number, a.linenum_attr.start};
         return true;
     }
     if (a.Is("relative")) {
-        if (!a.CheckType(EAttrType::STRING, chart->Error)) return true;
-        //if EAttrType::CLEAR, we are OK above and a.value is ""
-        rel = {a.value, a.linenum_attr.start};
+        if (a.type == EAttrType::CLEAR || a.value.empty())
+            chart->Error.Error(a, true, "Missing entity name.");
+        else if (a.CheckType(EAttrType::STRING, chart->Error))
+            rel = {a.value, a.linenum_attr.start};
         return true;
     }
     if (a.Is("collapsed")) {
@@ -405,27 +401,34 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
     std::unique_ptr<EntityAppHelper> ret = std::make_unique<EntityAppHelper>();
     //If we have children, add them to "ret->entities"
     ret->target = name; //this will be the target for any note after us.
-    if (ch) {
-        for (auto j = ch->begin(); j!=ch->end(); /*nope*/) {
-            EntityCommand * const ce = dynamic_cast<EntityCommand *>(j->get());
-            Note * const cn = dynamic_cast<Note *>(j->get());
-            if (ce) {
+    if (ch)
+        for (auto j = ch->begin(); j!=ch->end(); /*nope*/)
+            if (EntityCommand* const ce = dynamic_cast<EntityCommand*>(j->get())) {
                 ce->MoveMyContentAfter(*ret);  //ce is emptied out of all EntityDefs and tmp stored notes
                 //set the target for notes following to the last child.
                 ret->target = ret->entities.back()->name;
                 j++;
-            } else if (cn!=nullptr) {
+            } else if (Note* const cn = dynamic_cast<Note*>(j->get())) {
                 //Ok, a note, assume it wants to point to us or our last child so far
                 ret->notes.emplace_back(std::unique_ptr<Note>(cn), ret->target);
                 j->release();
-                ch->erase(j++);
+                j = ch->erase(j);
             } else {
                 chart->Error.Error((*j)->file_pos.start, "Only entity definitions are allowed here. Ignoring this.");
                 j++;
             }
-        }
-        delete ch;
+    delete ch;
+
+    //Remove children we are not defining
+    const auto bads = std::ranges::remove_if(ret->entities, [](std::unique_ptr<EntityApp>& e) {return !e->defining; });
+    for (auto &bad : bads) {
+        EntityRef ei = chart->AllEntities.Find_by_Name(bad->name);
+        _ASSERT(ei);
+        chart->Error.Error(bad->file_pos.start, "Cannot make an already existing entity part of a group.",
+                            "Entity remains outside this group.");
+        chart->Error.Error(ei->file_pos, bad->file_pos.start, "Entity '" + ei->name + "' was defined here.");
     }
+    ret->entities.erase(bads.begin(), bads.end());
 
     //Check that we apply certain attributes the right way for grouped entities
     if (ret->entities.size()) {
@@ -467,15 +470,15 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
             EntityRef rel_entity = chart->AllEntities.Find_by_Name(*rel);
             if (rel_entity == nullptr) {
                 if (chart->pedantic) {
-                    string s = "Cound not find entity '" + *rel;
-                    s += "' in attribute 'relative'. Ignoring attriute.";
+                    string s = "Could not find entity '" + *rel;
+                    s += "' in attribute 'relative'. Ignoring attribute.";
                     chart->Error.Error(rel.file_pos, s);
                 } else {
                     std::unique_ptr<EntityApp> ed = std::make_unique<EntityApp>(rel->c_str(), chart);
                     ed->AddAttributeList(nullptr, nullptr, rel.file_pos);
                     chart->AutoGenEntities.Append(std::move(ed));
                     //position remains at the old, not incremented Entity_max_pos
-                    //the definedHere of the entityDef above will be set to true in chart->PostParseProcees
+                    //the definedHere of the entityDef above will be set to true in chart->PostParseProcess
                 }
             } else if (rel_entity->children_names.size()) {
                 //the entity in the "relative" attribute is a grouped one
@@ -516,7 +519,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
             else
                 style_name = "entity";
 
-            //Detemine a proper starting style, which is the default style and text
+            //Determine a proper starting style, which is the default style and text
             //Do not add the contents of "style" (from attributes), those will be applied
             //at EntityCommand->PostParseProcess() to the running_style
             //(which is initialized with what we specify here)
@@ -534,7 +537,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
             //we use the value from the context (true by default)
             if (!style_to_use.read().indicator)
                 style_to_use.write().indicator = chart->MyCurrentContext().indicator.value();
-            //Copy shapeand shape size from 'style' (where set by attributes) to 'style_to_use'
+            //Copy shape and shape size from 'style' (where set by attributes) to 'style_to_use'
             //which contains the default shape.
             //Note that all other attributes will be reflected via the 'running_style' mechanism,
             //so we need not copy them, but this is a const member of Entity and we need it in
