@@ -165,7 +165,6 @@ std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view
 #ifdef BUILD_GUI
 TextureID TextureFromSurface(cairo_surface_t *rgba_surface);
 void DestroyTexture(ImTextureID t);
-int MscGenKeyMap[MscGenKey_COUNT];
 const char directory_separator = '/';
 void GUIBeep() { printf("\a"); }
 
@@ -435,19 +434,6 @@ int main(int argc, char* argv[])
         // Setup Platform/Renderer backends
         ImGui_ImplSDL2_InitForOpenGL(window, gl_context);
         ImGui_ImplOpenGL3_Init(glsl_version);
-        MscGenKeyMap[MscGenKey_F2] = SDL_SCANCODE_F2;
-        MscGenKeyMap[MscGenKey_F3] = SDL_SCANCODE_F3;
-        MscGenKeyMap[MscGenKey_F4] = SDL_SCANCODE_F4;
-        MscGenKeyMap[MscGenKey_F5] = SDL_SCANCODE_F5;
-        MscGenKeyMap[MscGenKey_F8] = SDL_SCANCODE_F8;
-        MscGenKeyMap[MscGenKey_S] = SDL_SCANCODE_S;
-        MscGenKeyMap[MscGenKey_N] = SDL_SCANCODE_N;
-        MscGenKeyMap[MscGenKey_W] = SDL_SCANCODE_W;
-        MscGenKeyMap[MscGenKey_O] = SDL_SCANCODE_O;
-        MscGenKeyMap[MscGenKey_R] = SDL_SCANCODE_R;
-        MscGenKeyMap[MscGenKey_F] = SDL_SCANCODE_F;
-        MscGenKeyMap[MscGenKey_H] = SDL_SCANCODE_H;
-        MscGenKeyMap[MscGenKey_E] = SDL_SCANCODE_E;
 
 #ifdef HAS_FONTCONFIG
         std::get<GUIInit>(ret).languages.cshParams->fontnames = GetFontNames();
